@@ -23,14 +23,15 @@ import sys
 import hashlib
 import urlparse
 import warnings
-from copy import deepcopy
+from copy import deepcopy, copy
+from functools import partial
 
 import logging
 log = logging.getLogger('mapproxy.config')
 
 from mapproxy.config import load_default_config, finish_base_config
 from mapproxy.config.spec import validate_mapproxy_conf
-from mapproxy.util import memoize
+from mapproxy.util.py import memoize
 from mapproxy.util.ext.odict import odict
 from mapproxy.util.yaml import load_yaml_file, YAMLError
 
@@ -38,9 +39,10 @@ class ConfigurationError(Exception):
     pass
 
 class ProxyConfiguration(object):
-    def __init__(self, conf, conf_base_dir=None, seed=False):
+    def __init__(self, conf, conf_base_dir=None, seed=False, renderd=False):
         self.configuration = conf
         self.seed = seed
+        self.renderd = renderd
 
         if conf_base_dir is None:
             conf_base_dir = os.getcwd()
@@ -55,7 +57,7 @@ class ProxyConfiguration(object):
 
     def load_globals(self, conf_base_dir):
         self.globals = GlobalConfiguration(conf_base_dir=conf_base_dir,
-                                           conf=self.configuration.get('globals', {}),
+                                           conf=self.configuration.get('globals') or {},
                                            context=self)
 
     def load_grids(self):
@@ -63,15 +65,16 @@ class ProxyConfiguration(object):
 
         self.grids['GLOBAL_GEODETIC'] = GridConfiguration(dict(srs='EPSG:4326', name='GLOBAL_GEODETIC'), context=self)
         self.grids['GLOBAL_MERCATOR'] = GridConfiguration(dict(srs='EPSG:900913', name='GLOBAL_MERCATOR'), context=self)
+        self.grids['GLOBAL_WEBMERCATOR'] = GridConfiguration(dict(srs='EPSG:3857', origin='nw', name='GLOBAL_WEBMERCATOR'), context=self)
 
-        for grid_name, grid_conf in self.configuration.get('grids', {}).iteritems():
+        for grid_name, grid_conf in (self.configuration.get('grids') or {}).iteritems():
             grid_conf.setdefault('name', grid_name)
             self.grids[grid_name] = GridConfiguration(grid_conf, context=self)
 
     def load_caches(self):
         self.caches = odict()
         caches_conf = self.configuration.get('caches')
-        if not caches_conf: return None # TODO config error
+        if not caches_conf: return
         if isinstance(caches_conf, list):
             caches_conf = list_of_dicts_to_ordered_dict(caches_conf)
         for cache_name, cache_conf in caches_conf.iteritems():
@@ -80,7 +83,7 @@ class ProxyConfiguration(object):
 
     def load_sources(self):
         self.sources = SourcesCollection()
-        for source_name, source_conf in self.configuration.get('sources', {}).iteritems():
+        for source_name, source_conf in (self.configuration.get('sources') or {}).iteritems():
             self.sources[source_name] = SourceConfiguration.load(conf=source_conf, context=self)
 
     def load_tile_layers(self):
@@ -201,10 +204,19 @@ class ProxyConfiguration(object):
         self.services = ServiceConfiguration(self.configuration.get('services', {}), context=self)
 
     def configured_services(self):
-        from mapproxy.util import local_base_config
-        with local_base_config(self.base_config):
+        with self:
             return self.services.services()
 
+    def __enter__(self):
+        # push local base_config onto config stack
+        import mapproxy.config.config
+        mapproxy.config.config._config.push(self.base_config)
+
+    def __exit__(self, type, value, traceback):
+        # pop local base_config from config stack
+        import mapproxy.config.config
+        mapproxy.config.config._config.pop()
+
     @property
     def base_config(self):
         return self.globals.base_config
@@ -247,6 +259,8 @@ class GridConfiguration(ConfigurationBase):
 
         if 'base' in self.conf:
             base_grid_name = self.conf['base']
+            if not base_grid_name in self.context.grids:
+                raise ConfigurationError('unknown base %s for grid %s' % (base_grid_name, self.conf['name']))
             conf = self.context.grids[base_grid_name].conf.copy()
             conf.update(self.conf)
             conf.pop('base')
@@ -299,6 +313,7 @@ class GlobalConfiguration(ConfigurationBase):
         finish_base_config(self.base_config)
 
         self.image_options = ImageOptionsConfiguration(self.conf.get('image', {}), context)
+        self.renderd_address = self.get_value('renderd.address')
 
     def _copy_conf_values(self, d, target):
         for k, v in d.iteritems():
@@ -350,6 +365,9 @@ class ImageOptionsConfiguration(ConfigurationBase):
                 conf['resampling'] = conf.pop('resampling_method')
             if 'encoding_options' in conf:
                 self._check_encoding_options(conf['encoding_options'])
+            if 'merge_method' in conf:
+                warnings.warn('merge_method now defaults to composite. option no longer required',
+                    DeprecationWarning)
             formats_config[format] = conf
         for format, conf in formats_config.iteritems():
             if 'format' not in conf and format.startswith('image/'):
@@ -372,6 +390,8 @@ class ImageOptionsConfiguration(ConfigurationBase):
 
     def image_opts(self, image_conf, format):
         from mapproxy.image.opts import ImageOptions
+        if not image_conf:
+            image_conf = {}
 
         conf = {}
         if format in self.formats:
@@ -386,12 +406,16 @@ class ImageOptionsConfiguration(ConfigurationBase):
         colors = image_conf.get('colors')
         mode = image_conf.get('mode')
         encoding_options = image_conf.get('encoding_options')
+        if 'merge_method' in image_conf:
+            warnings.warn('merge_method now defaults to composite. option no longer required',
+                DeprecationWarning)
 
         self._check_encoding_options(encoding_options)
 
         # only overwrite default if it is not None
         for k, v in dict(transparent=transparent, opacity=opacity, resampling=resampling,
-            format=img_format, colors=colors, mode=mode, encoding_options=encoding_options).iteritems():
+            format=img_format, colors=colors, mode=mode, encoding_options=encoding_options,
+        ).iteritems():
             if v is not None:
                 conf[k] = v
 
@@ -451,7 +475,9 @@ class SourcesCollection(dict):
                 " tagged sources only supported for WMS/Mapserver/Mapnik" % key)
 
         uses_req = source.conf.get('type') != 'mapnik'
-        source = deepcopy(source)
+
+        source = copy(source)
+        source.conf = deepcopy(source.conf)
 
         if uses_req:
             supported_layers = source.conf['req'].get('layers', [])
@@ -468,6 +494,7 @@ class SourcesCollection(dict):
             source.conf['req']['layers'] = layers
         else:
             source.conf['layers'] = layers
+
         return source
 
     def __contains__(self, key):
@@ -571,7 +598,7 @@ class WMSSourceConfiguration(SourceConfiguration):
             url = prefix + context.globals.abspath(url[7:])
         lg_client = WMSLegendURLClient(url)
         legend_cache = LegendCache(cache_dir=cache_dir)
-        return WMSLegendSource([lg_client], legend_cache)
+        return WMSLegendSource([lg_client], legend_cache, static=True)
 
     def fi_xslt_transformer(self, conf, context):
         from mapproxy.featureinfo import XSLTransformer, has_xslt_support
@@ -585,7 +612,7 @@ class WMSSourceConfiguration(SourceConfiguration):
         return fi_transformer
 
     def image_opts(self, format=None):
-        if 'transparent' not in self.conf.get('image', {}):
+        if 'transparent' not in (self.conf.get('image') or {}):
             transparent = self.conf['req'].get('transparent')
             if transparent is not None:
                 transparent = bool(str(transparent).lower() == 'true')
@@ -632,7 +659,7 @@ class WMSSourceConfiguration(SourceConfiguration):
         coverage = self.coverage()
         res_range = resolution_range(self.conf)
 
-        transparent_color = self.conf.get('image', {}).get('transparent_color')
+        transparent_color = (self.conf.get('image') or {}).get('transparent_color')
         transparent_color_tolerance = self.context.globals.get_value(
             'image.transparent_color_tolerance', self.conf)
         if transparent_color:
@@ -706,7 +733,7 @@ class WMSSourceConfiguration(SourceConfiguration):
             version = self.conf.get('wms_opts', {}).get('version', '1.1.1')
             lg_req = self.conf['req'].copy()
             lg_clients = []
-            lg_layers = lg_req['layers'].split(',')
+            lg_layers = str(lg_req['layers']).split(',')
             del lg_req['layers']
             for lg_layer in lg_layers:
                 lg_req['layer'] = lg_layer
@@ -779,13 +806,23 @@ class MapnikSourceConfiguration(SourceConfiguration):
         mapfile = self.context.globals.abspath(self.conf['mapfile'])
 
         if self.conf.get('use_mapnik2', False):
-            from mapproxy.source.mapnik import Mapnik2Source as MapnikSource, mapnik2 as mapnik_api
-        else:
-            from mapproxy.source.mapnik import MapnikSource, mapnik as mapnik_api
+            warnings.warn('use_mapnik2 option is no longer needed for Mapnik 2 support',
+                DeprecationWarning)
+
+        from mapproxy.source.mapnik import MapnikSource, mapnik as mapnik_api
         if mapnik_api is None:
             raise ConfigurationError('Could not import Mapnik, please verify it is installed!')
+
+        if self.context.renderd:
+            # only renderd guarantees that we have a single proc/thread
+            # that accesses the same mapnik map object
+            reuse_map_objects = True
+        else:
+            reuse_map_objects = False
+
         return MapnikSource(mapfile, layers=layers, image_opts=image_opts,
-            coverage=coverage, res_range=res_range, lock=lock)
+            coverage=coverage, res_range=res_range, lock=lock,
+            reuse_map_objects=reuse_map_objects)
 
 class TileSourceConfiguration(SourceConfiguration):
     supports_meta_tiles = False
@@ -848,6 +885,7 @@ source_configuration_types = {
 class CacheConfiguration(ConfigurationBase):
     defaults = {'format': 'image/png', 'grids': ['GLOBAL_MERCATOR']}
 
+    @memoize
     def cache_dir(self):
         cache_dir = self.conf.get('cache', {}).get('directory')
         if cache_dir:
@@ -859,6 +897,12 @@ class CacheConfiguration(ConfigurationBase):
         return self.context.globals.get_path('cache_dir', self.conf,
             global_key='cache.base_dir')
 
+    def lock_dir(self):
+        lock_dir = self.context.globals.get_path('cache.tile_lock_dir', self.conf)
+        if not lock_dir:
+            lock_dir = os.path.join(self.cache_dir(), 'tile_locks')
+        return lock_dir
+
     def _file_cache(self, grid_conf, file_ext):
         from mapproxy.cache.file import FileCache
 
@@ -878,8 +922,14 @@ class CacheConfiguration(ConfigurationBase):
 
         lock_timeout = self.context.globals.get_value('http.client_timeout', {})
 
-        return FileCache(cache_dir, file_ext=file_ext, directory_layout=directory_layout,
-            lock_timeout=lock_timeout, link_single_color_images=link_single_color_images)
+        return FileCache(
+            cache_dir,
+            lock_dir=self.lock_dir(),
+            file_ext=file_ext,
+            directory_layout=directory_layout,
+            lock_timeout=lock_timeout,
+            link_single_color_images=link_single_color_images,
+        )
 
     def _mbtiles_cache(self, grid_conf, file_ext):
         from mapproxy.cache.mbtiles import MBTilesCache
@@ -892,13 +942,37 @@ class CacheConfiguration(ConfigurationBase):
             mbfile_path = self.context.globals.abspath(filename)
         else:
             mbfile_path = os.path.join(self.cache_dir(), filename)
-        return MBTilesCache(mbfile_path)
+
+        return MBTilesCache(
+            mbfile_path,
+            lock_dir=self.lock_dir(),
+        )
+
+    def _sqlite_cache(self, grid_conf, file_ext):
+        from mapproxy.cache.mbtiles import MBTilesLevelCache
+
+        cache_dir = self.conf.get('cache', {}).get('directory')
+        if cache_dir:
+            cache_dir = os.path.join(
+                self.context.globals.abspath(cache_dir),
+                grid_conf.tile_grid().name
+            )
+        else:
+            cache_dir = self.cache_dir()
+            cache_dir = os.path.join(
+                cache_dir,
+                self.conf['name'],
+                grid_conf.tile_grid().name
+            )
+
+        return MBTilesLevelCache(
+            cache_dir,
+            lock_dir=self.lock_dir(),
+        )
 
     def _couchdb_cache(self, grid_conf, file_ext):
         from mapproxy.cache.couchdb import CouchDBCache, CouchDBMDTemplate
 
-        cache_dir = self.cache_dir()
-
         db_name = self.conf['cache'].get('db_name')
         if not db_name:
             suffix = grid_conf.conf['srs'].replace(':', '')
@@ -912,9 +986,40 @@ class CacheConfiguration(ConfigurationBase):
         tile_id = self.conf['cache'].get('tile_id')
 
         return CouchDBCache(url=url, db_name=db_name,
-            lock_dir=cache_dir, file_ext=file_ext, tile_grid=grid_conf.tile_grid(),
+            lock_dir=self.lock_dir(), file_ext=file_ext, tile_grid=grid_conf.tile_grid(),
             md_template=md_template, tile_id_template=tile_id)
 
+    def _riak_cache(self, grid_conf, file_ext):
+        from mapproxy.cache.riak import RiakCache
+
+        default_ports = self.conf['cache'].get('default_ports', {})
+        default_pb_port = default_ports.get('pb', 8087)
+        default_http_port = default_ports.get('http', 8098)
+
+        nodes = self.conf['cache'].get('nodes')
+        if not nodes:
+            nodes = [{'host': '127.0.0.1'}]
+
+        for n in nodes:
+            if 'pb_port' not in n:
+                n['pb_port'] = default_pb_port
+            if 'http_port' not in n:
+                n['http_port'] = default_http_port
+
+        protocol = self.conf['cache'].get('protocol', 'pbc')
+        bucket = self.conf['cache'].get('bucket')
+        if not bucket:
+            suffix = grid_conf.tile_grid().name
+            bucket = self.conf['name'] + '_' + suffix
+
+        use_secondary_index = self.conf['cache'].get('secondary_index', False)
+
+        return RiakCache(nodes=nodes, protocol=protocol, bucket=bucket,
+            tile_grid=grid_conf.tile_grid(),
+            lock_dir=self.lock_dir(),
+            use_secondary_index=use_secondary_index,
+        )
+
     def _tile_cache(self, grid_conf, file_ext):
         if self.conf.get('disable_storage', False):
             from mapproxy.cache.dummy import DummyCache
@@ -950,7 +1055,21 @@ class CacheConfiguration(ConfigurationBase):
                 image_opts.format = ImageFormat('image/png')
         return image_opts
 
-    def source(self, params=None, tile_grid=None):
+    def supports_tiled_only_access(self, params=None, tile_grid=None):
+        caches = self.caches()
+        if len(caches) > 1:
+            return False
+
+        cache_grid, extent, tile_manager = caches[0]
+        image_opts = self.image_opts()
+
+        if (tile_grid.is_subset_of(cache_grid)
+            and params.get('format') == image_opts.format):
+            return True
+
+        return False
+
+    def source(self, params=None, tile_grid=None, tiled_only=False):
         from mapproxy.source.tile import CacheSource
         from mapproxy.layer import map_extent_from_grid
 
@@ -962,21 +1081,19 @@ class CacheConfiguration(ConfigurationBase):
             return source
 
         cache_grid, extent, tile_manager = caches[0]
-        if tile_grid.is_subset_of(cache_grid):
-            tiled_only = True
-        else:
-            tiled_only = False
+        image_opts = self.image_opts()
 
         cache_extent = map_extent_from_grid(tile_grid)
         cache_extent = extent.intersection(cache_extent)
 
         source = CacheSource(tile_manager, extent=cache_extent,
-            image_opts=self.image_opts(), tiled_only=tiled_only)
+            image_opts=image_opts, tiled_only=tiled_only)
         return source
 
     @memoize
     def caches(self):
         from mapproxy.cache.tile import TileManager
+        from mapproxy.cache.base import TileLocker
         from mapproxy.image.opts import compatible_image_options
         from mapproxy.layer import map_extent_from_grid, merge_layer_extents
 
@@ -984,6 +1101,11 @@ class CacheConfiguration(ConfigurationBase):
         if self.conf.get('format') == 'mixed' and not self.conf.get('request_format') == 'image/png':
             raise ConfigurationError('request_format must be set to image/png if mixed mode is enabled')
         request_format = self.conf.get('request_format') or self.conf.get('format')
+        if '/' in request_format:
+            request_format_ext = request_format.split('/', 1)[1]
+        else:
+            request_format_ext = request_format
+
         caches = []
 
         meta_buffer = self.context.globals.get_value('meta_buffer', self.conf,
@@ -995,17 +1117,40 @@ class CacheConfiguration(ConfigurationBase):
         concurrent_tile_creators = self.context.globals.get_value('concurrent_tile_creators', self.conf,
             global_key='cache.concurrent_tile_creators')
 
+        renderd_address = self.context.globals.get_value('renderd.address', self.conf)
+
         for grid_name, grid_conf in self.grid_confs():
             sources = []
             source_image_opts = []
+
+            # a cache can directly access source tiles when _all_ sources are caches too
+            # and when they have compatible grids by using tiled_only on the CacheSource
+            # check if all sources support tiled_only
+            tiled_only = True
+            for source_name in self.conf['sources']:
+                if source_name in self.context.sources:
+                    tiled_only = False
+                    break
+                elif source_name in self.context.caches:
+                    cache_conf = self.context.caches[source_name]
+                    tiled_only = cache_conf.supports_tiled_only_access(
+                        params={'format': request_format},
+                        tile_grid=grid_conf.tile_grid(),
+                    )
+                    if not tiled_only:
+                        break
+
             for source_name in self.conf['sources']:
                 if source_name in self.context.sources:
                     source_conf = self.context.sources[source_name]
                     source = source_conf.source({'format': request_format})
                 elif source_name in self.context.caches:
                     cache_conf = self.context.caches[source_name]
-                    source = cache_conf.source({'format': request_format},
-                        tile_grid=grid_conf.tile_grid())
+                    source = cache_conf.source(
+                        params={'format': request_format},
+                        tile_grid=grid_conf.tile_grid(),
+                        tiled_only=tiled_only,
+                    )
                 else:
                     raise ConfigurationError('unknown source %s' % source_name)
                 if source:
@@ -1019,12 +1164,37 @@ class CacheConfiguration(ConfigurationBase):
             tile_filter = self._tile_filter()
             image_opts = compatible_image_options(source_image_opts, base_opts=base_image_opts)
             cache = self._tile_cache(grid_conf, image_opts.format.ext)
+            identifier = self.conf['name'] + '_' + tile_grid.name
+
+            tile_creator_class = None
+
+            if not self.context.renderd and renderd_address:
+                from mapproxy.cache.renderd import RenderdTileCreator, has_renderd_support
+                if not has_renderd_support():
+                    raise ConfigurationError("renderd requires Python >=2.6 and requests")
+                if self.context.seed:
+                    priority = 10
+                else:
+                    priority = 100
+
+                cache_dir = self.cache_dir()
+
+                lock_dir = self.context.globals.get_value('cache.tile_lock_dir')
+                if not lock_dir:
+                    lock_dir = os.path.join(cache_dir, 'tile_locks')
+
+                lock_timeout = self.context.globals.get_value('http.client_timeout', {})
+                locker = TileLocker(lock_dir, lock_timeout, identifier + '_renderd')
+                tile_creator_class = partial(RenderdTileCreator, renderd_address,
+                    priority=priority, tile_locker=locker)
             mgr = TileManager(tile_grid, cache, sources, image_opts.format.ext,
-                              image_opts=image_opts,
+                              image_opts=image_opts, identifier=identifier,
+                              request_format=request_format_ext,
                               meta_size=meta_size, meta_buffer=meta_buffer,
                               minimize_meta_requests=minimize_meta_requests,
                               concurrent_tile_creators=concurrent_tile_creators,
-                              pre_store_filter=tile_filter)
+                              pre_store_filter=tile_filter,
+                              tile_creator_class=tile_creator_class)
             extent = merge_layer_extents(sources)
             if extent.is_default:
                 extent = map_extent_from_grid(tile_grid)
@@ -1149,9 +1319,21 @@ class LayerConfiguration(ConfigurationBase):
                          sources, fi_sources, lg_sources, res_range=res_range, md=self.conf.get('md'))
         return layer
 
+    @memoize
+    def dimensions(self):
+        from mapproxy.layer import Dimension
+        dimensions = {}
+
+        for dimension, conf in self.conf.get('dimensions', {}).iteritems():
+            values = [str(val) for val in  conf.get('values', ['default'])]
+            default = conf.get('default', values[-1])
+            dimensions[dimension.lower()] = Dimension(dimension, values, default=default)
+        return dimensions
+
     @memoize
     def tile_layers(self):
         from mapproxy.service.tile import TileLayer
+        from mapproxy.cache.dummy import DummyCache
 
         sources = []
         for source_name in self.conf.get('sources', []):
@@ -1167,9 +1349,19 @@ class LayerConfiguration(ConfigurationBase):
         if len(sources) > 1:
             return []
 
+        dimensions = self.dimensions()
+
         tile_layers = []
         for cache_name in sources:
             for grid, extent, cache_source in self.context.caches[cache_name].caches():
+
+                if dimensions and not isinstance(cache_source.cache, DummyCache):
+                    # caching of dimension layers is not supported yet
+                    raise ConfigurationError(
+                        "caching of dimension layer (%s) is not supported yet."
+                        " need to `disable_storage: true` on %s cache" % (self.conf['name'], cache_name)
+                    )
+
                 md = {}
                 md['title'] = self.conf['title']
                 md['name'] = self.conf['name']
@@ -1177,9 +1369,10 @@ class LayerConfiguration(ConfigurationBase):
                 md['grid_name'] = grid.name
                 md['name_internal'] = md['name_path'][0] + '_' + md['name_path'][1]
                 md['format'] = self.context.caches[cache_name].image_opts().format
+                md['cache_name'] = cache_name
                 md['extent'] = extent
                 tile_layers.append(TileLayer(self.conf['name'], self.conf['title'],
-                                             md, cache_source))
+                                             md, cache_source, dimensions=dimensions))
 
         return tile_layers
 
@@ -1283,7 +1476,6 @@ class ServiceConfiguration(ConfigurationBase):
             if template and '{{' in template:
                 # TODO remove warning in 1.6
                 log.warn("double braces in WMTS restful_template are deprecated {{x}} -> {x}")
-
             services.append(WMTSRestServer(layers, md, template=template,
                 max_tile_age=max_tile_age))
 
@@ -1309,12 +1501,7 @@ class ServiceConfiguration(ConfigurationBase):
                                                        global_key='wms.image_formats')
         image_formats = {}
         for format in image_formats_names:
-            if format.startswith('image/'):
-                from mapproxy.image.opts import ImageOptions
-                opts = ImageOptions(format=format)
-            else:
-                # lookup custom format
-                opts = self.context.globals.image_options.image_opts({}, format)
+            opts = self.context.globals.image_options.image_opts({}, format)
             if opts.format in image_formats:
                 log.warn('duplicate mime-type for WMS image_formats: "%s" already configured',
                     opts.format)
@@ -1354,11 +1541,20 @@ class ServiceConfiguration(ConfigurationBase):
         tile_layers = self.tile_layers(conf)
         image_formats = self.context.globals.get_value('image_formats', conf, global_key='wms.image_formats')
         srs = self.context.globals.get_value('srs', conf, global_key='wms.srs')
+
+        # WMTS restful template
+        wmts_conf = self.context.services.conf.get('wmts', {})
+        from mapproxy.service.wmts import WMTSRestServer
+        if wmts_conf:
+            restful_template = wmts_conf.get('restful_template', WMTSRestServer.default_template)
+        else:
+            restful_template = WMTSRestServer.default_template
+
         return DemoServer(layers, md, tile_layers=tile_layers,
-            image_formats=image_formats, srs=srs, services=services)
+            image_formats=image_formats, srs=srs, services=services, restful_template=restful_template)
 
 
-def load_configuration(mapproxy_conf, seed=False, ignore_warnings=True):
+def load_configuration(mapproxy_conf, seed=False, ignore_warnings=True, renderd=False):
     conf_base_dir = os.path.abspath(os.path.dirname(mapproxy_conf))
 
     try:
@@ -1370,7 +1566,8 @@ def load_configuration(mapproxy_conf, seed=False, ignore_warnings=True):
         log.warn(error)
     if not informal_only or (errors and not ignore_warnings):
         raise ConfigurationError('invalid configuration')
-    return ProxyConfiguration(conf_dict, conf_base_dir=conf_base_dir, seed=seed)
+    return ProxyConfiguration(conf_dict, conf_base_dir=conf_base_dir, seed=seed,
+        renderd=renderd)
 
 def load_configuration_file(files, working_dir):
     """
