@@ -50,7 +50,8 @@ image_opts = {
     'format': str(),
     'encoding_options': {
         anything(): anything()
-    }
+    },
+    'merge_method': str(),
 }
 
 http_opts = {
@@ -84,14 +85,26 @@ source_commons = combined(
     }
 )
 
+riak_node = {
+    'host': str(),
+    'pb_port': number(),
+    'http_port': number(),
+}
+
 cache_types = {
     'file': {
         'directory_layout': str(),
         'use_grid_names': bool(),
         'directory': str(),
+        'tile_lock_dir': str(),
+    },
+    'sqlite': {
+        'directory': str(),
+        'tile_lock_dir': str(),
     },
     'mbtiles': {
-        'filename': str()
+        'filename': str(),
+        'tile_lock_dir': str(),
     },
     'couchdb': {
         'url': str(),
@@ -100,7 +113,18 @@ cache_types = {
             anything(): anything()
         },
         'tile_id': str(),
+        'tile_lock_dir': str(),
     },
+    'riak': {
+        'nodes': [riak_node],
+        'protocol': one_of('pbc', 'http', 'https'),
+        'bucket': str(),
+        'default_ports': {
+            'pb': number(),
+            'http': number(),
+        },
+        'secondary_index': bool(),
+    }
 }
 
 on_error = {
@@ -176,6 +200,15 @@ grid_opts = {
     'threshold_res': [number()],
 }
 
+ogc_service_md = {
+    'title': basestring,
+    'abstract': basestring,
+    'online_resource': basestring,
+    'contact': anything(),
+    'fees': basestring,
+    'access_constraints': basestring,
+}
+
 mapproxy_yaml_spec = {
     'globals': {
         'image': {
@@ -188,11 +221,13 @@ mapproxy_yaml_spec = {
                 anything(): image_opts,
             },
             'font_dir': str(),
+            'merge_method': str(),
         },
         'http': http_opts,
         'cache': {
             'base_dir': str(),
             'lock_dir': str(),
+            'tile_lock_dir': str(),
             'meta_size': [number()],
             'meta_buffer': number(),
             'max_tile_limit': number(),
@@ -211,6 +246,9 @@ mapproxy_yaml_spec = {
             'expires_hours': number(),
         },
         'mapserver': mapserver_opts,
+        'renderd': {
+            'address': str(),
+        }
     },
     'grids': {
         anything(): grid_opts,
@@ -255,6 +293,7 @@ mapproxy_yaml_spec = {
             'kvp': bool(),
             'restful': bool(),
             'restful_template': str(),
+            'md': ogc_service_md,
         },
         'wms': {
             'srs': [str()],
@@ -270,14 +309,7 @@ mapproxy_yaml_spec = {
             'on_source_errors': str(),
             'max_output_pixels': one_of(number(), [number()]),
             'strict': bool(),
-            'md': {
-                'title': basestring,
-                'abstract': basestring,
-                'online_resource': basestring,
-                'contact': anything(),
-                'fees': basestring,
-                'access_constraints': basestring,
-            },
+            'md': ogc_service_md,
         },
     },
 
@@ -369,8 +401,17 @@ mapproxy_yaml_spec = {
             'legendurl': str(),
             'layers': recursive(),
             'md': wms_130_layer_md,
+            'dimensions': {
+                anything(): {
+                    required('values'): [one_of(basestring, float, int)],
+                    'default': one_of(basestring, float, int),
+                }
+            }
         })])
     ),
+     # `parts` can be used for partial configurations that are referenced
+     # from other sections (e.g. coverages, dimensions, etc.)
+    'parts': anything(),
 }
 
 if __name__ == '__main__':
