@@ -45,7 +45,7 @@ def app_factory(global_options, config_dir, allow_listing=False, **local_options
     """
     return make_wsgi_app(config_dir, asbool(allow_listing))
 
-def make_wsgi_app(config_dir, allow_listing=True):
+def make_wsgi_app(config_dir, allow_listing=True, debug=False):
     """
     Create a MultiMapProxy with the given config directory.
 
@@ -55,16 +55,17 @@ def make_wsgi_app(config_dir, allow_listing=True):
     """
     config_dir = os.path.abspath(config_dir)
     loader = DirectoryConfLoader(config_dir)
-    return MultiMapProxy(loader, list_apps=allow_listing)
+    return MultiMapProxy(loader, list_apps=allow_listing, debug=debug)
 
 
 class MultiMapProxy(object):
 
-    def __init__(self, loader, list_apps=False, app_cache_size=100):
+    def __init__(self, loader, list_apps=False, app_cache_size=100, debug=False):
         self.loader = loader
         self.list_apps = list_apps
         self._app_init_lock = Lock()
         self.apps = LRU(app_cache_size)
+        self.debug = debug
 
     def __call__(self, environ, start_response):
         req = Request(environ)
@@ -73,7 +74,7 @@ class MultiMapProxy(object):
     def handle(self, req):
         app_name = req.pop_path()
         if not app_name:
-            return self.index_list()
+            return self.index_list(req)
 
         if not app_name or (
                 app_name not in self.apps and not self.loader.app_available(app_name)
@@ -84,16 +85,17 @@ class MultiMapProxy(object):
         req.environ['mapproxy.instance_name'] = app_name
         return self.proj_app(app_name)
 
-    def index_list(self):
+    def index_list(self, req):
         """
         Return greeting response with a list of available apps (if enabled with list_apps).
         """
         import mapproxy.version
         html = "<html><body><h1>Welcome to MapProxy %s</h1>" % mapproxy.version.version
 
+        url = req.script_url
         if self.list_apps:
             html += "<h2>available instances:</h2><ul>"
-            html += '\n'.join('<li><a href="%(name)s/">%(name)s</a></li>' % {'name': app}
+            html += '\n'.join('<li><a href="%(url)s/%(name)s/">%(name)s</a></li>' % {'url': url, 'name': app}
                               for app in self.loader.available_apps())
             html += '</ul>'
         html += '</body></html>'
@@ -112,11 +114,12 @@ class MultiMapProxy(object):
 
         if not proj_app:
             with self._app_init_lock:
+                proj_app, timestamp = self.apps.get(proj_name, (None, None))
                 if self.loader.needs_reload(proj_name, timestamp):
                     proj_app, m_time = self.create_app(proj_name)
                     self.apps[proj_name] = proj_app, m_time
                 else:
-                    proj_app = self.apps[proj_name]
+                    proj_app, timestamp = self.apps[proj_name]
 
         return proj_app
 
@@ -127,7 +130,7 @@ class MultiMapProxy(object):
         mapproxy_conf = self.loader.app_conf(proj_name)['mapproxy_conf']
         m_time = os.path.getmtime(mapproxy_conf)
         log.info('initializing project app %s with %s', proj_name, mapproxy_conf)
-        app = make_mapproxy_wsgi_app(mapproxy_conf)
+        app = make_mapproxy_wsgi_app(mapproxy_conf, debug = self.debug)
         return app, m_time
 
 
