@@ -13,23 +13,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-
-import platform
-
 __all__ = ['LockTimeout', 'FileLock', 'LockError', 'cleanup_lockdir']
 
-if platform.system() == "Java":
-    from mapproxy.platform.jython.lock import (
-        LockTimeout,
-        FileLock,
-        LockError,
-        cleanup_lockdir,
-    )
-    LockTimeout, FileLock, LockError, cleanup_lockdir # prevent pyflakes warnings
-else:
-    from mapproxy.platform.cpython.lock import (
-        LockTimeout,
-        FileLock,
-        LockError,
-        cleanup_lockdir)
-    LockTimeout, FileLock, LockError, cleanup_lockdir # prevent pyflakes warnings
+from mapproxy.platform.cpython.lock import (
+    LockTimeout,
+    FileLock,
+    LockError,
+    cleanup_lockdir)
+LockTimeout, FileLock, LockError, cleanup_lockdir # prevent pyflakes warnings
