@@ -86,6 +86,9 @@ def serve_multiapp_develop_command(args):
     parser.add_option("-b", "--bind",
                       dest="address", default='127.0.0.1:8080',
                       help="Server socket [127.0.0.1:8080]")
+    parser.add_option("--debug", default=False, action='store_true',
+                      dest="debug",
+                      help="Enable debug mode")
     options, args = parser.parse_args(args)
 
     if len(args) != 2:
@@ -97,10 +100,18 @@ def serve_multiapp_develop_command(args):
 
     host, port = parse_bind_address(options.address)
 
+    if options.debug and host not in ('localhost', '127.0.0.1'):
+        print textwrap.dedent("""\
+        ################# WARNING! ##################
+        Running debug mode with non-localhost address
+        is a serious security vulnerability.
+        #############################################\
+        """)
+
     setup_logging()
     from mapproxy.multiapp import make_wsgi_app
     from mapproxy.util.ext.serving import run_simple
-    app = make_wsgi_app(mapproxy_conf_dir)
+    app = make_wsgi_app(mapproxy_conf_dir, debug=options.debug)
 
     run_simple(host, port, app, use_reloader=True, processes=1,
         threaded=True, passthrough_errors=True)
@@ -218,7 +229,8 @@ class CreateCommand(object):
 
         template_dir = self.template_dir()
 
-        for filename in ('mapproxy.yaml', 'seed.yaml'):
+        for filename in ('mapproxy.yaml', 'seed.yaml',
+            'full_example.yaml', 'full_seed_example.yaml'):
             to = os.path.join(outdir, filename)
             from_ = os.path.join(template_dir, filename)
             if os.path.exists(to) and not self.options.force:
@@ -316,7 +328,7 @@ def print_items(data, title='Commands'):
 
 def main():
     parser = NonStrictOptionParser("usage: %prog COMMAND [options]",
-        version='MapProxy ' + version, add_help_option=False)
+        add_help_option=False)
     options, args = parser.parse_args()
 
     if len(args) < 1 or args[0] in ('--help', '-h'):
@@ -325,6 +337,10 @@ def main():
         print_items(commands)
         sys.exit(1)
 
+    if len(args) == 1 and args[0] == '--version':
+        print 'Mapproxy ' + version
+        sys.exit(1)
+
     command = args[0]
     if command not in commands:
         parser.print_help()
