@@ -16,14 +16,15 @@
 from __future__ import with_statement, division
 import sys
 from contextlib import contextmanager
+import time
 
 from mapproxy.config import base_config
 from mapproxy.grid import MetaGrid
 from mapproxy.source import SourceError
-from mapproxy.util import local_base_config
+from mapproxy.config import local_base_config
 from mapproxy.util.ext.itertools import izip_longest
 from mapproxy.util.lock import LockTimeout
-from mapproxy.seed.util import format_seed_task
+from mapproxy.seed.util import format_seed_task, timestamp
 from mapproxy.seed.cachelock import DummyCacheLocker, CacheLockedError
 
 from mapproxy.seed.util import (exp_backoff, ETA, limit_sub_bbox,
@@ -46,11 +47,40 @@ else:
     queue_class = multiprocessing.Queue
 
 
-class TileWorkerPool(object):
+class TileProcessor(object):
+    def __init__(self, dry_run=False):
+        self._lastlog = time.time()
+        self.dry_run = dry_run
+
+    def log_progress(self, progress):
+        if (self._lastlog + .1) < time.time():
+            # log progress at most every 100ms
+            print '[%s] %6.2f%% %s \tETA: %s\r' % (
+                timestamp(), progress[1]*100, progress[0],
+                progress[2]
+            ),
+            sys.stdout.flush()
+            self._lastlog = time.time()
+
+    def process(self, tiles, progress):
+        if not self.dry_run:
+            self.process_tiles(tiles)
+
+        self.log_progress(progress)
+
+    def stop(self):
+        raise NotImplementedError()
+
+    def process_tiles(self, tiles):
+        raise NotImplementedError()
+
+
+class TileWorkerPool(TileProcessor):
     """
     Manages multiple TileWorker.
     """
     def __init__(self, task, worker_class, size=2, dry_run=False, progress_logger=None):
+        TileProcessor.__init__(self, dry_run=dry_run)
         self.tiles_queue = queue_class(size)
         self.task = task
         self.dry_run = dry_run
@@ -221,30 +251,35 @@ class TileWalker(object):
                 pass
         self.report_progress(self.task.levels[0], self.task.coverage.bbox)
 
-    def _walk(self, cur_bbox, levels, all_subtiles=False):
+    def _walk(self, cur_bbox, levels, current_level=0, all_subtiles=False):
         """
         :param cur_bbox: the bbox to seed in this call
         :param levels: list of levels to seed
         :param all_subtiles: seed all subtiles and do not check for
                              intersections with bbox/geom
         """
-        current_level, levels = levels[0], levels[1:]
         bbox_, tiles, subtiles = self.grid.get_affected_level_tiles(cur_bbox, current_level)
         total_subtiles = tiles[0] * tiles[1]
-
         if len(levels) < self.skip_geoms_for_last_levels:
             # do not filter in last levels
             all_subtiles = True
         subtiles = self._filter_subtiles(subtiles, all_subtiles)
 
-        if current_level <= self.report_till_level:
+        if current_level in levels and current_level <= self.report_till_level:
             self.report_progress(current_level, cur_bbox)
 
         if not self.seed_progress.running():
-            self.report_progress(current_level, cur_bbox)
+            if current_level in levels:
+                self.report_progress(current_level, cur_bbox)
             self.tile_mgr.cleanup()
             raise StopProcess()
 
+        process = False;
+        if current_level in levels:
+            levels = levels[1:]
+            process = True            
+        current_level += 1
+
         for i, (subtile, sub_bbox, intersection) in enumerate(subtiles):
             if subtile is None: # no intersection
                 self.seed_progress.step_forward(total_subtiles)
@@ -260,9 +295,12 @@ class TileWalker(object):
                     if self.seed_progress.already_processed():
                         self.seed_progress.step_forward()
                     else:
-                        self._walk(sub_bbox, levels,
+                        self._walk(sub_bbox, levels, current_level=current_level,
                             all_subtiles=all_subtiles)
 
+            if not process:
+                continue
+
             if not self.work_on_metatiles:
                 # collect actual tiles
                 handle_tiles = self.grid.tile_list(subtile)
@@ -332,7 +370,6 @@ class SeedTask(object):
         if self.coverage.intersects(bbox, self.grid.srs): return INTERSECTS
         return NONE
 
-
 class CleanupTask(object):
     """
     :param coverage: area for the cleanup
