@@ -17,6 +17,7 @@ from __future__ import with_statement, division
 import os
 import re
 import sys
+import shutil
 from mapproxy.platform.image import Image
 import functools
 
@@ -292,7 +293,30 @@ class TestWMS111(WMSTest):
 
         assert validate_with_dtd(xml, 'wms/1.1.1/exception_1_1_1.dtd')
 
+    def test_direct_layer_non_image_response(self):
+        self.common_map_req.params['layers'] = 'direct'
+        expected_req = ({'path': r'/service?LAYERs=bar&SERVICE=WMS&FORMAT=image%2Fpng'
+                          '&REQUEST=GetMap&HEIGHT=200&SRS=EPSG%3A4326&styles='
+                          '&VERSION=1.1.1&BBOX=-180.0,0.0,0.0,80.0'
+                          '&WIDTH=200'},
+                            {'body': 'notanimage', 'headers': {'content-type': 'image/jpeg'}})
+        with mock_httpd(('localhost', 42423), [expected_req]):
+            resp = self.app.get(self.common_map_req)
+            eq_(resp.content_type, 'application/vnd.ogc.se_xml')
+            xml = resp.lxml
+            eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code'), [])
+            assert 'error while processing image file' in \
+                 xml.xpath('//ServiceException/text()')[0]
+
+            assert validate_with_dtd(xml, 'wms/1.1.1/exception_1_1_1.dtd')
+
     def test_get_map(self):
+        # check custom tile lock directory
+        tiles_lock_dir = os.path.join(test_config['base_dir'], 'wmscachetilelockdir')
+        # make sure custom tile_lock_dir was not created by other tests
+        shutil.rmtree(tiles_lock_dir, ignore_errors=True)
+        assert not os.path.exists(tiles_lock_dir)
+
         self.created_tiles.append('wms_cache_EPSG900913/01/000/000/001/000/000/001.jpeg')
         with tmp_image((256, 256), format='jpeg') as img:
             expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fjpeg'
@@ -306,6 +330,28 @@ class TestWMS111(WMSTest):
                 assert 35000 < int(resp.headers['Content-length']) < 75000
                 eq_(resp.content_type, 'image/png')
 
+        # check custom tile_lock_dir
+        assert os.path.exists(tiles_lock_dir)
+
+    def test_get_map_non_image_response(self):
+        self.created_tiles.append('wms_cache_EPSG900913/01/000/000/001/000/000/001.jpeg')
+        expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fjpeg'
+                                  '&REQUEST=GetMap&HEIGHT=256&SRS=EPSG%3A900913&styles='
+                                  '&VERSION=1.1.1&BBOX=0.0,0.0,20037508.3428,20037508.3428'
+                                  '&WIDTH=256'},
+                        {'body': 'notanimage', 'headers': {'content-type': 'image/jpeg'}})
+        with mock_httpd(('localhost', 42423), [expected_req]):
+            self.common_map_req.params['bbox'] = '0,0,180,90'
+            resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'application/vnd.ogc.se_xml')
+        xml = resp.lxml
+        print resp.body
+        eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code'), [])
+        assert 'unable to transform image: cannot identify image file' in \
+             xml.xpath('//ServiceException/text()')[0]
+
+        assert validate_with_dtd(xml, 'wms/1.1.1/exception_1_1_1.dtd')
+
     def test_get_map_direct_fwd_params_layer(self):
         img = create_tmp_image((200, 200), format='png')
         expected_req = ({'path': r'/service?LAYERs=bar&SERVICE=WMS&FORMAT=image%2Fpng'
@@ -368,6 +414,12 @@ class TestWMS111(WMSTest):
         is_111_exception(resp.lxml, 'Could not transform BBOX: Invalid result.')
 
     def test_get_map100(self):
+        # check global tile lock directory
+        tiles_lock_dir = os.path.join(test_config['base_dir'], 'defaulttilelockdir')
+        # make sure global tile_lock_dir was ot created by other tests
+        shutil.rmtree(tiles_lock_dir, ignore_errors=True)
+        assert not os.path.exists(tiles_lock_dir)
+
         self.created_tiles.append('wms_cache_100_EPSG900913/01/000/000/001/000/000/001.jpeg')
         # request_format tiff, cache format jpeg, wms request in png
         with tmp_image((256, 256), format='tiff') as img:
@@ -382,6 +434,9 @@ class TestWMS111(WMSTest):
                 resp = self.app.get(self.common_map_req)
                 eq_(resp.content_type, 'image/png')
 
+        # check global tile lock directory was created
+        assert os.path.exists(tiles_lock_dir)
+
     def test_get_map130(self):
         self.created_tiles.append('wms_cache_130_EPSG900913/01/000/000/001/000/000/001.jpeg')
         with tmp_image((256, 256), format='jpeg') as img:
@@ -415,9 +470,10 @@ class TestWMS111(WMSTest):
         expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fpng'
                                   '&REQUEST=GetFeatureInfo&HEIGHT=200&SRS=EPSG%3A900913'
                                   '&VERSION=1.1.1&BBOX=1000.0,400.0,2000.0,1400.0&styles='
-                                  '&WIDTH=200&QUERY_LAYERS=foo,bar&X=10&Y=20'},
+                                  '&WIDTH=200&QUERY_LAYERS=foo,bar&X=10&Y=20&feature_count=100'},
                         {'body': 'info', 'headers': {'content-type': 'text/plain'}})
         with mock_httpd(('localhost', 42423), [expected_req]):
+            self.common_fi_req.params['feature_count'] = 100
             resp = self.app.get(self.common_fi_req)
             eq_(resp.content_type, 'text/plain')
             eq_(resp.body, 'info')
