@@ -25,7 +25,7 @@ from mapproxy.config.loader import (
 from mapproxy.cache.tile import TileManager
 from mapproxy.test.helper import TempFile
 from mapproxy.test.unit.test_grid import assert_almost_equal_bbox
-from nose.tools import eq_
+from nose.tools import eq_, assert_raises
 from nose.plugins.skip import SkipTest
 
 class TestLayerConfiguration(object):
@@ -518,6 +518,66 @@ sources:
                 assert False, 'expected configuration error'
 
 
+class TestConfImport(object):
+
+    yaml_string = """
+globals:
+  http:
+    client_timeout: 1
+    headers:
+      baz: quux
+"""
+
+    yaml_parent = """
+globals:
+  http:
+    client_timeout: 2
+    headers:
+      foo: bar
+      bar: qux
+      baz: qax
+"""
+
+    yaml_grand_parent = """
+globals:
+  http:
+    client_timeout: 3
+    method: GET
+    headers:
+      bar: baz
+"""
+    
+    def test_loading(self):
+        with TempFile() as gp:
+            open(gp, 'w').write(self.yaml_grand_parent)
+            self.yaml_parent = """
+base:
+  - %s
+%s
+""" % (gp, self.yaml_parent)
+
+            with TempFile() as p:
+                open(p, 'w').write(self.yaml_parent)
+
+                self.yaml_string = """
+base: [%s]
+%s
+""" % (p, self.yaml_string)
+
+                with TempFile() as cfg:
+                    open(cfg, 'w').write(self.yaml_string)
+
+        
+                    config = load_configuration(cfg)
+        
+                    http = config.globals.get_value('http')
+                    eq_(http['client_timeout'], 1)
+                    eq_(http['headers']['bar'], 'qux')
+                    eq_(http['headers']['foo'], 'bar')
+                    eq_(http['headers']['baz'], 'quux')
+                    eq_(http['method'], 'GET')
+
+
 class TestConfMerger(object):
     def test_empty_base(self):
         a = {'a': 1, 'b': [12, 13]}
@@ -544,6 +604,18 @@ class TestConfMerger(object):
         eq_({'a': {'aa': 12, 'ab': 13, 'a':{'aaa': 100, 'aab': 101}}}, m)
 
 
+class TestLoadConfiguration(object):
+    def test_with_warnings(object):
+        with TempFile() as f:
+            open(f, 'w').write("""
+services:
+  unknown:
+                """)
+            load_configuration(f) # defaults to ignore_warnings=True
+            
+            assert_raises(ConfigurationError, load_configuration, f, ignore_warnings=False)
+
+
 class TestImageOptions(object):
     def test_default_format(self):
         conf_dict = {
@@ -777,4 +849,4 @@ class TestImageOptions(object):
         conf = ProxyConfiguration(conf_dict)
         
         conf.globals.image_options.image_opts({}, 'image/jpeg')
-        
\ No newline at end of file
+        
