@@ -1201,3 +1201,17 @@ class TestGridSubset(object):
 
         assert not g1.is_subset_of(g2)
 
+    def test_non_matching_bboxfor_origins(self):
+        g1 = tile_grid(SRS(21781), bbox=[420000, 30000, 900000, 360000],
+            res=[250], origin='nw')
+        g2 = tile_grid(SRS(21781), bbox=[420000, 30000, 900000, 360000],
+            res=[250], origin='sw')
+
+        assert not g1.is_subset_of(g2)
+
+    def test_no_tile_errors(self):
+        # g1 is not a subset, check that we don't get any NoTile errors
+        g1 = tile_grid(SRS(3857), res=[100000, 50000, 10000, 1000, 100, 10, 1, 0.5])
+        g2 = tile_grid(SRS(3857), res=[100, 1])
+
+        assert not g1.is_subset_of(g2)
